#version 330 core

in vec3 WorldPosition;
in vec2 Texcoords;
in vec3 WorldNormal;

uniform sampler2D color_texture;
uniform sampler2D bumpmap_texture;
uniform vec4 color;

uniform mat3 normalMatrix;
uniform mat4 viewMatrix;

uniform bool hasColorTexture = true;
uniform bool hasBumpMapTexture = true;
uniform bool bumpMappingEnabled = true;

layout (location = 0) out vec4 gWorldPosition;
layout (location = 1) out vec4 gWorldNormal;
layout (location = 2) out vec4 gDiffuse;


vec3 calculateBumpMappingNormal(vec3 normal)
{
	// Create Tangent Space
	vec3 up = vec3(0,1,0);
	vec3 tangent = normalize(cross(up, normal));
	if(length(tangent) == 0)
		tangent = vec3(1, 0, 0);
	vec3 bitangent = normalize(cross(normal, tangent));

	mat3 tbnMatrix = mat3(tangent, bitangent, normal);
	vec3 bumpNormal = normalize(texture(bumpmap_texture, Texcoords).xyz * 2.0 - 1.0);

	return normalize(tbnMatrix * bumpNormal);
}


void main()
{
    gWorldPosition.xyz = WorldPosition;
	gWorldNormal.xyz = normalize(WorldNormal);
	gDiffuse = (hasColorTexture) ? texture(color_texture, Texcoords) : color;
	
	if(hasBumpMapTexture == true && bumpMappingEnabled)	
       gWorldNormal.xyz = calculateBumpMappingNormal(WorldNormal);    
		
	gWorldPosition.a = 1;
	gWorldNormal.a = 1;
}