#version 330 core

in vec2 texUV;

uniform sampler2D color_texture;
uniform vec4 color;
uniform bool hasColorTexture;

uniform float bloomHighpassFilterFactor = 0.90f;

layout(location = 0) out vec4 fragColor;
layout(location = 1) out vec4 brightColor;


vec4 bloomHighPassFilter(vec4 color)
{
	// Do a Highpass Filter for Bloom Effect
	// Convert to grayscale using NTSC conversion weights
    float brightness = dot(color.rgb, vec3(0.299, 0.587, 0.114));
	return (brightness > bloomHighpassFilterFactor) ?  color : vec4(0);	
}

void main()
{
	fragColor = (hasColorTexture) ? texture(color_texture, texUV) : color;
	fragColor.a *= 0.7;
	brightColor = bloomHighPassFilter(fragColor);
}