#version 330 core

#define NUM_SAMPLES 128
#define Num_SAMPLES_RPC 0.0078125

#define TAU 0.0001
#define PHI 10000000.0
#define PI_RCP 0.31830988618379067153776752674503

in vec2 texcoords;


layout(location = 0) out vec4 fragColor;

vec3 executeRaymarching(vec3 VLI)
{
 	rayPositionLightVS.xyz += stepSize * invViewDirLightVS.xyz ;

//calculate the shadowtherm => we need a shadowmap

 	// Fetch whether the current position on the ray is visible form the light's perspective - or not
 	float3 shadowTerm = getShadowTerm ( shadowMapSampler, shadowMapSamplerState, rayPositionLightSS.xyz ).xxx ;
 	// Distance to the current position on the ray in light view-space
 	float d = length ( rayPositionLightVS.xyz ) ;
 	float dRcp = rcp ( d ) ;
 	// Calculate the final light contribution for the sample on the ray...
 	float3 intens = TAU * ( shadowTerm * (PHI * 0.25 * PI_RCP) * dRcp * dRcp ) * exp( -d * TAU ) * exp ( -l * TAU ) * stepSize ;
 	// ... and add it to the total contribution of the ray
  	VLI += intens ;

	return VLI;
}



void main()
{
   	float raymarchDistanceLimit = 999999.0;

	 // Reduce noisyness by truncating the starting position
	 float raymarchDistance = trunc ( clamp ( length ( cameraPositionLightVS.xyz - positionLightVS.xyz ) , 0.0, raymarchDistanceLimit ) ) ;
 
	// Calculate the size of each step
 	float stepSize = raymarchDistance * NUM_SAMPLES_RCP ;
 	float3 rayPositionLightVS = positionLightVS.xyz ;
 
	// The total light contribution accumulated along the ray
 	float3 VLI = 0.0 ;
 
	// ... start the actual ray marching
	for ( float l = raymarchDistance; l > stepSize ; l -= stepSize )
 	{
 		VLI = executeRaymarching(VLI) ;
 	}

	fragColor = light_color_diffuse.rgb * VLI ;
}

//references 
//http://bglatzel.movingblocks.net/wp-content/uploads/2014/05/Volumetric-Lighting-for-Many-Lights-in-Lords-of-the-Fallen-With-Notes.pdf
//file:///C:/Users/User/Downloads/ShaderX3-Advanced%20Rendering%20with%20DirectX%20and%20OpenGL.pdf
//https://research.nvidia.com/sites/default/files/publications/McGuireI3D11Shadows.pdf